import os
import sys
from unittest import TestCase
from unittest.mock import patch, MagicMock

from dagger import DaggerAPI, initDagger, initLambda
from dagger.constants import DAGGER_URL

@patch('dagger.DaggerAPI.requests')
class DaggerTestCase(TestCase):
    def tearDown(self):
        if 'AWS_REGION' in os.environ:
            del os.environ['AWS_REGION']
        if '_HANDLER' in os.environ:
            del os.environ['_HANDLER']

    @patch('dagger.DaggerAPI')
    def test_initDagger_lambda(self, mock_api, requests):
        os.environ['_HANDLER'] = 'test'

        mock_bootstrap = MagicMock()
        sys.modules['bootstrap'] = mock_bootstrap

        mock_event = MagicMock()
        mock_context = MagicMock()

        def _handle_event_request(lambda_runtime_client, request_handler, *args, **kwargs):
            request_handler(mock_event, mock_context)
        mock_bootstrap.handle_event_request = _handle_event_request

        initDagger('testkey')

        mock_api.assert_called_once()

        mock_request_handler = MagicMock()
        mock_bootstrap.handle_event_request(MagicMock(), mock_request_handler)

        mock_request_handler.assert_called_once_with(mock_event, mock_context)

        assert len(mock_api.return_value.createTask.mock_calls) == 2
        assert len(mock_api.return_value.createTask.return_value.update.mock_calls) == 1



import os
import unittest
from unittest.mock import patch, MagicMock

from dagger.integrations.aws_ecs import wrapMainFunction

TEST_EXECUTION_ENV = {
    'DockerId': 'docker-id',
    'Name': 'container-name', 
    'DockerName': 'docker-name', 
    'Image': 'docker-image', 
    'ImageID': 'image-id', 
    'Labels': {
        'com.amazonaws.ecs.cluster': 'test-cluster', 
        'com.amazonaws.ecs.container-name': 'test-container-name', 
        'com.amazonaws.ecs.task-arn': 'test-task-arn/1234', 
        'com.amazonaws.ecs.task-definition-family': 'test-task-definition-family', 
        'com.amazonaws.ecs.task-definition-version': 'test-task-definition-version'
    },
    'DesiredStatus': 'RUNNING', 
    'KnownStatus': 'RUNNING', 
    'Limits': {'CPU': 0, 'Memory': 2048}, 
    'CreatedAt': '2020-06-07T16:07:45.321115096Z', 
    'StartedAt': '2020-06-07T16:08:10.522444681Z', 
    'Type': 'NORMAL', 
    'Networks': [{'NetworkMode': 'awsvpc', 'IPv4Addresses': ['10.0.21.154']}]
}

def mock_requests(url):
    mock_response = MagicMock()
    mock_response.json.return_value = TEST_EXECUTION_ENV
    return mock_response

@patch('dagger.integrations.aws_ecs.requests.get', side_effect=mock_requests)
class TestAWSECSIntegration(unittest.TestCase):
    def tearDown(self):
        remove_env_variables = ['AWS_REGION', 'ECS_CONTAINER_METADATA_URI']
        for env_variable_name in remove_env_variables:
            if env_variable_name in os.environ:
                del os.environ[env_variable_name]

    @patch('dagger.integrations.aws_ecs.__main__')
    def test_wrapFunction(self, mock_main, mock_request_get):
        mock_main.__file__ = 'test'

        mock_dagger_api = MagicMock()
        mock_main_func = MagicMock()

        wrapped_func = wrapMainFunction(mock_dagger_api, mock_main_func)

        mock_input = MagicMock()
        wrapped_func(mock_input, asdf=mock_input)

        mock_main_func.assert_called_with(mock_input, asdf=mock_input)

        mock_dagger_api.createTask.assert_called_with(
            mock_main.__file__, 
            TEST_EXECUTION_ENV['Labels']['com.amazonaws.ecs.task-arn'], 
            task_input={
                'args': (mock_input, ), 
                'kwargs': {'asdf': mock_input}
            },
            task_metadata={
                'region': None,
                'execution_env': None,
                'logGroupName': '/ecs/test-task-definition-family',
                'logStreamName': 'ecs/test-container-name/1234',
                'ecs_container_metadata': TEST_EXECUTION_ENV
            }
        )



#!/usr/bin/env python

from setuptools import setup, find_packages

# The text of the README file
with open( 'Readme.md' ) as f:
    README = f.read()

setup(
    name='python-dagger',
    version='0.0.10',
    description='Api for python',
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/dagger-team/python-dagger",
    author='Mike Lyons',
    author_email='mdl0394@gmail.com',
    packages=find_packages( exclude=[ 'tmp', 'examples', 'build' ] ),
    scripts=[],
    install_requires=[
        'requests'
    ],
    license="MIT",
)



import os

import requests

from dagger.constants import DAGGER_URL
from dagger.Task import TaskListener, Task

class DaggerAPI(TaskListener):
    def __init__(self, api_token, verbose=True):
        self.api_token = api_token
        self.verbose = verbose

    def createTask(self, task_name, task_run_id, initial_status='started', **update_kwargs):
        return Task(self, task_name, task_run_id).update(task_status=initial_status, **update_kwargs)

    def sendTaskStatus(self, status, task_name, task_run_id, task_input, task_output, task_metadata):
        body = dict(
            status=status,
            task_name=task_name,
            id=task_run_id,
            input=dict(input=task_input),
            output=dict(output=task_output),
            metadata=task_metadata,
            api_token=self.api_token
        )

        if self.verbose:
            print('Dagger request')
            print(DAGGER_URL, body)

        response = requests.post(
            DAGGER_URL,
            json=body
        )

        if self.verbose:
            print('Dagger response')
            print(response)
            print(response.content)

    def onTaskUpdate(self, task):
        self.sendTaskStatus(
            task.task_status,
            task.task_name,
            task.task_run_id,
            task.task_input,
            task.task_output,
            task.task_metadata
        )
